unit u_formats;

{$mode objfpc}{$H+}

interface

uses
  StrUtils, Math;

// Fmt('xyD', Value: longint);
// Fmt('xyDd', Value: double);
// TechFmt('xyDdE', Value: double);
// TechFmt gives exponent in multiples of 3
// or with literals like m for milli, M for Mega, etc.
//
// Format specifier:
// x:
//   +: always sign
//   -: only sign when negative, space at +
//   n: no space for a sign
// y:
//   0: leading zeros
//   space: leading spaces
// D: {1..9}
//   number of digits before decimal point
// decimal character is always point, no thousend separators
// d: {0..9}
//   number of decimals. ignored for integers
// E: use E notation in stead of literals

// Output:
function Fmt(Formatspec: string; Value: longint): shortstring;
function Fmt(Formatspec: string; Value: double): shortstring;
function TechFmt(Formatspec: string; Value: double): shortstring;
// Input:
function FIval (InpStr: string): longint;
function FRval (InpStr: string): double;

implementation

function Fmt(FormatSpec: string; Value: Longint): shortstring;
var
	TempVal, i: Longint;
	TempStr: shortstring;
	nDigits: integer;
begin
	nDigits:= FIVal(midstr(FormatSpec, 3, 1));
	TempVal := abs(Value);
	str(TempVal:nDigits, TempStr);
	if midstr(FormatSpec, 2, 1) = '0' then
		for i := 1 to length (TempStr) do
			if TempStr[i] = ' ' then TempStr[i]:= '0';
  case midstr(FormatSpec, 1, 1) of
  	'+': if Value < 0 then TempStr := '-' + TempStr else TempStr := '+' + TempStr;
  	'-': if Value < 0 then TempStr := '-' + TempStr else TempStr := ' ' + TempStr;
  end;
	Fmt := TempStr;
end;

function Fmt(FormatSpec: string; Value: Double): shortstring;
var
	i: integer;
	TempVal: Double;
	TempStr: shortstring;
	nDigits: integer;
	nDecimals: integer;
begin
	nDigits:= FIVal(midstr(FormatSpec, 3, 1));
	nDecimals:= FIVal(midstr(FormatSpec, 4, 1));
  if nDecimals > 0 then inc(nDigits);
	TempVal:= abs(Value);
	str(TempVal:nDigits+nDecimals:nDecimals, TempStr);
	if midstr(FormatSpec, 2, 1) = '0' then
		for i := 1 to length (TempStr) do
			if TempStr[i] = ' ' then TempStr[i]:= '0';
	case midstr(FormatSpec, 1, 1) of
		'+': if Value < 0 then TempStr := '-' + TempStr else TempStr := '+' + TempStr;
		'-': if Value < 0 then TempStr := '-' + TempStr else TempStr := ' ' + TempStr;
	end;
	Fmt := TempStr;
end;

function TechFmt(FormatSpec: string; Value: Double): shortstring;
var
 	Mantisse: real;
 	i, Sign: Integer;
 	FmtStr, TempStr: string;
   UseLiterals: boolean;
const Suffixes = 'afpnum kMGTPE';
begin
  UseLiterals:= (midstr(FormatSpec, 5, 1)) <> 'E';
 	Sign := 1;
 	If Value < 0 Then //catch negatives
  begin
 		Sign:= -1;
    Value:= -Value;
 	end;
  Mantisse:= Value;
  i := 0;
  if Mantisse <> 0 then // catch zero
  begin
   	while Mantisse >= 1000 do
   	begin
     	Mantisse := Mantisse / 1000;
     	inc(i);
   	end;
   	while Mantisse < 1 do
   	begin
     	Mantisse := Mantisse * 1000;
     	dec(i);
   	end;
  end;
  TempStr:= Fmt(FormatSpec, Mantisse * Sign);
  if (i * 3 < -18) or (i * 3 > 18) or (UseLiterals = false) then
    TechFmt:= TempStr + 'E' + Fmt('+02', i * 3)
 	else
    TechFmt:= TempStr + Midstr(Suffixes, i + 7, 1);
end;

function FIval(InpStr: string): longint;
var Temp: longint;
begin
  val(InpStr, Temp);
  FIval:= Temp;
end;

function FRval(InpStr: string): double;
var Temp: real;
begin
	val(InpStr, Temp);
	FRval:= Temp;
end;

end.

