unit u_HiPadGUI;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, SdpoSerial, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls;

type

  { TFrmHiPadGui }

  TFrmHiPadGui = class(TForm)
    ButtNew: TButton;
    ButtSaveAs: TButton;
    ButDelete: TButton;
    ButClose: TButton;
    ButtSelectCommPort: TButton;
    CbxStripType: TCheckBox;
    CbxGcode: TCheckBox;
    CbxAutoRefresh: TCheckBox;
    Memo: TMemo;
    TbxCommPort: TEdit;
    Label4: TLabel;
    SaveDialog: TSaveDialog;
    CommPort: TSdpoSerial;
    TbxNumPoints: TEdit;
    TbxFileName: TEdit;
    Timer1: TTimer;
    procedure ButCloseClick(Sender: TObject);
    procedure ButDeleteClick(Sender: TObject);
    procedure ButtNewClick(Sender: TObject);
    procedure ButtSaveAsClick(Sender: TObject);
    procedure ButtSelectCommPortClick(Sender: TObject);
    procedure CbxGcodeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CommPortRxData(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure OpenCommPort;
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  FrmHiPadGui: TFrmHiPadGui;

// frowards
procedure InitPlot;
function MsgBox: boolean;

implementation
{$R *.lfm}
uses strutils, LCLtype, u_formats, u_selectcomport;

const ConfigFileName = 'HiPadConf.txt';
type tDrec = record act:char; aX, aY: real; end;
var
  CommPortName: string;
  OutputFileName: string;
  Xzero, Yzero : real;
  Datarray: array[0 .. 10000] of tDrec;
  Index: integer;
  Gcode: boolean;

procedure PlotCross (cX, cY: integer; Clor: longint);
Const r = 4;
  var SaveColor: longint;
begin
  with FrmHiPadGUI do
  begin
  SaveColor:= Canvas.Pen.Color;
  Canvas.Pen.Color:= Clor;
  Canvas.Line(cX-r, cY-r, cX+r, cY+r);
  Canvas.Line(cX+r, cY-r, cX-r, cY+r);
  Canvas.Pen.Color:= SaveColor;
  end;
end;

procedure Plot;
var n, XX, XXX, YY, YYY: integer;
begin
  InitPlot;
  with FrmHiPadGUI do
  begin
  	Memo.clear;
    TbxNumPoints.text := Fmt('n 6', Index);
  	for n := 0 to Index-1 do
   		begin
    		// plotarea = 700 x 700 so /4*10 from digitizer range 270 x 270 mm
      	XX:=  10 + round((Datarray[n-1].aX + Xzero + 5.5) / 4 * 10);
      	YY:= 710 - round((Datarray[n-1].aY + Yzero + 5.5) / 4 * 10);
      	XXX:=  10 + round((Datarray[n].aX + Xzero + 5.5) / 4 * 10);
      	YYY:= 710 - round((Datarray[n].aY + Yzero + 5.5) / 4 * 10);
       	if XX < 15 then XX := 15; if XX > 705 then XX := 705; // clip
				if YY < 15 then YY := 15; if YY > 705 then YY := 705;
      	if XXX < 15 then XXX := 15; if XXX > 705 then XXX := 705; // clip
				if YYY < 15 then YYY := 15; if YYY > 705 then YYY := 705;
     		if n = 0 then
      	begin
      		Canvas.line(XXX, YYY, XXX, YYY);
       		PlotCross(XXX, YYY, clred);
      	end
      	else
      	begin
       	  Canvas.line(XX, YY, XXX, YYY);
        	if n = Index-1 then PlotCross (XXX, YYY, clyellow);
      	end;
        with Datarray[n] do Memo.append(Fmt('+031',aX)+ ' '+ Fmt('+031',aY)+ ' '+act);
    	end;
  end;
end;

procedure TFrmHiPadGui.CommPortRxData(Sender: TObject);
const Data: string = '';
var Xin, Yin: real; Ctyp: char;
begin
  Data := Data + CommPort.ReadData;
  if length (Data) >= 15 then
  begin
    Xin := FIval(midstr(Data, 2, 6))/10;
    Yin := FIval(midstr(Data, 8, 6))/10;
    Ctyp := Data[1];
  	Data := '';
    if Index = 0 then
    begin
      Xzero := Xin;
      Yzero := Yin;
    end;
    with Datarray[Index] do
    begin
    	aX := Xin - Xzero;
    	aY := Yin - Yzero;
      act := Ctyp;
    end;
    inc(Index);
    Plot;
  end;
end;

function MsgBox: boolean;
var
  Reply, BoxStyle: Integer;
begin
  BoxStyle := MB_ICONQUESTION + MB_YESNO;
  Reply := Application.MessageBox('This will erase all your data',
  																'Attention', BoxStyle);
  if Reply = IDYES then MsgBox:= true else msgBox:= false;
end;

procedure InitPlot;
begin
	with FrmHiPadGUI.Canvas do
	begin
		Brush.Color:= clBlack;
		Pen.Color:= clwhite;
		Font.Color:= clWhite;
		Pen.Style := psSolid;
	 	Rectangle(10, 10, 710, 710);
		AutoReDraw:= true;
	end;
end;

procedure TFrmHiPadGui.ButtSaveAsClick(Sender: TObject);
var OutFile: text;
    n: integer;
begin
 	SaveDialog.InitialDir := GetCurrentDir;
	SaveDialog.Filter := 'Digitizer files|*.txt';
	SaveDialog.FilterIndex := 1;
	if SaveDialog.Execute then
	begin
		OutputFileName := SaveDialog.FileName;
    if ExtractFileExt(OutputFileName) <> '.txt' then OutputFileName := OutputFileName + '.txt';
		TbxFileName.text := '...\' + ExtractFileName(OutputFileName);
    system.assign(OutFile, OutputFileName);
    rewrite(OutFile);
    for n:= 0 to Index-1 do
      if CbxStripType.checked then
      begin
        if Gcode then
        	writeln (OutFile, 'G1 X'+ Fmt('+031', Datarray[n].aX)+
          									  ' Y'+ Fmt('+031', Datarray[n].aY))
        else
    			writeln (OutFile, Fmt('+031', Datarray[n].aX)+' '+
													  Fmt('+031', Datarray[n].aY));
        end
    	else
      	writeln (OutFile, Fmt('+031', Datarray[n].aX)+' '+
													Fmt('+031', Datarray[n].aY)+' '+ Datarray[n].act);
      end;
    system.close(OutFile);
end;

procedure TFrmHiPadGui.ButDeleteClick(Sender: TObject);
begin
	if Index > 0 then dec(Index);
  Plot;
end;

procedure TFrmHiPadGui.ButtNewClick(Sender: TObject);
begin
  if not msgbox then exit;
  Index := 0;
  Plot;
end;

procedure TFrmHiPadGui.ButCloseClick(Sender: TObject);
begin
  Datarray[Index] := Datarray[0];
  inc(Index);
  Plot;
end;

procedure TFrmHiPadGui.CbxGcodeChange(Sender: TObject);
begin
  Gcode := CbxGcode.Checked;
  CbxStripType.Enabled := not Gcode;
  if Gcode then
  begin
    CbxStripType.Checked := true;
    CbxStripType.Enabled := false;
  end;
end;

procedure TFrmHiPadGui.Timer1Timer(Sender: TObject);
begin
  if CbxAutoRefresh.Checked then Plot;
end;

procedure TFrmHiPadGui.OpenCommPort;
begin
   TbxCommPort.text := CommPortName;
   CommPort.Device := CommPortName;
   CommPort.BaudRate := br__4800;
   CommPort.DataBits:= db7bits;
   CommPort.Parity:= pNone;
   CommPort.StopBits:= sbTwo;
   CommPort.Device := CommPortName;
   CommPort.Open;
end;

procedure TFrmHiPadGui.ButtSelectCommPortClick(Sender: TObject);
var CommPortNumber: integer;
begin
  FrmSelectComPort.show;
  CommPort.close;
  FrmSelectComPort.Select(CommPortNumber);
  CommPortName := 'Com' + DelSpace(Fmt('n 2',CommPortNumber));
  OpenCommPort;
  FrmSelectComPort.hide;
end;

procedure WriteConfig;
var ConfigFile: text;
begin
  assign(ConfigFile, ConfigFileName);
  rewrite(ConfigFile);
  writeln(ConfigFile, CommPortName);
  close(ConfigFile);
end;

procedure ReadConfig;
var ConfigFile: text;
begin
  if not FileExists(ConfigFileName) then exit;
  assign(ConfigFile, ConfigFileName);
  reset(ConfigFile);
  readln(ConfigFile, CommPortName);
  close(ConfigFile);
  FrmHiPadGui.OpenCommPort;
end;

procedure TFrmHiPadGui.FormDestroy(Sender: TObject);
begin
	WriteConfig;
end;

procedure TFrmHiPadGui.FormCreate(Sender: TObject);
begin
	ReadConfig;
end;

end.

