unit u_numstrings;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StrUtils;

function BytesView(InpStr: string; Nch: integer): string;
function StrLittleToLong(InpStr: string):longint;
function StrLittleToSignedInt(InpStr: string):integer;
function LongLittleToString(InpLong: longint):string;
function IntLittleToString(InpInt: integer):string;

implementation

{ convert a string to a row of byte values }
function BytesView(InpStr: string; Nch: integer): string;
var N, A: integer; Ts, Tt: string;
begin
Ts := ''; Tt := '';
for N := 1 to Nch do
  begin
  A := byte(InpStr[N]);
  str(A:4, Ts);
  Tt := Tt + Ts + ':';
  end;
BytesView := Tt;
end;

{ convert string with Little Endian longint to longint }
function StrLittleToLong(inpstr: string):longint;
var StringOrLong: record
  	case boolean of
      	true: (dummy, Lint: longint); // takes 8 bytes
      	false: (Sint: string[7]); // also 8 bytes
  	end;
begin
// note string[0] is length byte
StringOrLong.Sint := '...' + InpStr; // InpStr in Sint[4..7] and len in [0]
StrLittleToLong := StringOrLong.Lint; // interprete bytes [4..7] as the longint
end;

{ convert string with Little Endian int16 to signed int16 }
function StrLittleToSignedInt(InpStr: string): integer;
var StringOrInt: record
  	case boolean of
      	true: (dummy, Lint: longint);
      	false: (Sint: string[7]);
  	end;
Temp: Longint;
begin
// note string[0] is length byte
StringOrInt.dummy := 0;
StringOrInt.Sint := '...' + InpStr;
Temp := StringOrInt.Lint;
if Temp > 32767 then Temp := Temp - longint(65536);
StrLittleToSignedInt := Temp;
end;

{ convert Little Endian longint32 to string }
function LongLittleToString(InpLong: longint): string;
var StringOrLong: record
  case boolean of
     true: (dummy, Lint: longint);
     false: (Sint: string[7]);
  end;
begin
// note string[0] is length byte
StringOrLong.Lint := InpLong;
LongLittleToString := midstr(StringOrLong.Sint,4,4);
end;

{ convert Little Endian int16 to string }
function IntLittleToString(InpInt: integer): string;
var StringOrInt: record
  case boolean of
    true: (dummy, NumInt: longint);
    false: (Sint: string[7]);
  end;
begin
// note string[0] is length byte
StringOrInt.NumInt := inpint;
StringOrInt.Sint[0] := char(7);
IntLittleToString := midstr(StringOrInt.Sint,4,2);
end;

end.

